from PyQt6.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton, 
                             QProgressBar, QPlainTextEdit, QScrollArea, QApplication)

# создание виджета фидбека, находится в правом нижнем углу
class FeedbackWidget(QWidget):
    def __init__(self, start_hidden=True):
        super().__init__()

        # создание кнопок переключения прогресс бара и окна с сообщениями
        # и размещение самих виджетов фидбека
        self.layout = QHBoxLayout(self)
        self.button_layout = QVBoxLayout()
        self.display_layout = QVBoxLayout()
        
        self.progress_messages_button = QPushButton('Progress Messages')
        self.progress_bar_button = QPushButton('Progress Bar')
        self.button_layout.addWidget(self.progress_messages_button)
        self.button_layout.addWidget(self.progress_bar_button)

        self.progress_bar = QProgressBar()
        self.progress_bar.hide()
        self.progress_bar.setValue(100)


            
        self.text_messages = QPlainTextEdit()
        self.text_messages.setReadOnly(True)
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidget(self.text_messages)
        self.scroll_area.setWidgetResizable(True)

        if start_hidden:
            self.progress_bar.hide()
            self.scroll_area.hide()

        self.display_layout.addWidget(self.progress_bar)
        self.display_layout.addWidget(self.scroll_area)

        self.layout.addLayout(self.button_layout)
        self.layout.addLayout(self.display_layout)

        self.progress_bar_button.clicked.connect(self.show_progress_bar)
        self.progress_messages_button.clicked.connect(self.show_progress_messages)

    # отобразить прогресс бар
    def show_progress_bar(self):
        self.progress_bar.show()
        self.scroll_area.hide()

    # отобразить сообщения о прогрессе от ядра
    def show_progress_messages(self):
        self.progress_bar.hide()
        self.scroll_area.show()

    # очистить область фидбека
    def reset(self):
        self.text_messages.clear()
        self.progress_bar.setValue(0)
        self.show_progress_bar()

    def update_progress(self, value):
        self.progress_bar.setValue(value)

    def append_message(self, message):
        self.text_messages.appendPlainText(message)

if __name__ == "__main__":
    app = QApplication([])
    widget = FeedbackWidget(start_hidden=False)
    widget.show()
    app.exec()
