import sys
from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QApplication, QWidget, QLineEdit, QVBoxLayout, QLabel
from static_list_widget import staticQLineEditor
from labeled_qline_widget import labeledQLineEditor

# создание новых ячеек для ввода параметров при нажатии на кнопку
# так называемый "динамический лист"
# для его создания импортируется статический лист
class dynamicQLineEditor(QWidget):
    def __init__(self, n_descr: str, element_descr: str):
        super().__init__()

        self.n_value = 0
        self.widget_list = []
        # значения, переданные пользователем в ячейку
        self.values = []
        self.is_created = False
        self.n_description = n_descr
        # описание, находящееся в ячейке, там написан тип данных
        self.element_description = element_descr

        # считывает количество создаваемых ячеек их View
        self.n_input = labeledQLineEditor(self.n_description, "int")

        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.addWidget(self.n_input)

    # это событие происходит, если была нажата кнопка
    def create_line_edits(self):
        if self.is_created:
            return

        # создание и добавление указанного количества полей
        self.n_value = self.n_input.get_value()
        len_of_static_list = 2
        labels_of_static_list = ["eps", "Thickness (nm)"]
        types_of_static_list = ["string", "float"]
        for i in range(self.n_value):
            el = staticQLineEditor(len_of_static_list, labels_of_static_list, types_of_static_list)
            self.widget_list.append(el)
            self.layout.addWidget(el)

        self.is_created = True

    # получение данных из поля для ввода
    def save_line_edit_value(self):
        self.values = []
        for i in range(self.n_value):
            self.values.append(self.widget_list[i].get_value())

    # сохранение данных из поля для ввода
    def get_value(self):
        self.save_line_edit_value()
        return self.values

    # реакция на нажатие кнопки
    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Return and not self.is_created:
            self.create_line_edits()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = dynamicQLineEditor("Enter N, pls", "Layer")
    window.show()
    sys.exit(app.exec())
