import sys
import subprocess
from PyQt6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget, QLabel, QLineEdit, QComboBox, QPushButton, \
    QGridLayout, QDialog, QScrollArea, QPlainTextEdit, QHBoxLayout, QFrame
from PyQt6 import QtGui
import pyqtgraph as pg
import json
import socket
import logging
from dynamic_list_widget import dynamicQLineEditor
from labeled_qline_widget import labeledQLineEditor
from labeledComboBoxEditor import labeledComboBoxWidget
from plotWidget import PlotWindow
from FeedbackWidget import FeedbackWidget

from datetime import datetime

# Настройка логгера
logging.basicConfig(filename='app_logs.log', level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

class MainWindow(QMainWindow):
    # здесь задаются основные и стартовые параметры для диалогового окна
    def __init__(self):
        super(MainWindow, self).__init__()

        # Настройка логгера для главного окна
        self.logger = logging.getLogger('main_window')
        self.logger.info("Initializing the main window")
        
        # заголовок главного окна
        self.setWindowTitle("Faculty of Physics")

# задаём стиль окна:
        
#         self.setStyleSheet("""
#     QMainWindow {
#         background-color: #203040;
#     }
#     QLabel {
#         color: white;
#     }
#     QPushButton {
#         background-color: #555555;
#         color: white;
#         border-radius: 5px;
#     }
#     QPushButton:hover {
#         background-color: #777777;
#     }
#     QProgressBar {
#         border: 2px solid #555555;
#         border-radius: 5px;
#         text-align: center;
#     }
#     QProgressBar::chunk {
#         background-color: #88C0D0;
#         width: 20px;
#     }
# """)

        # создание менюбар
        self.menu_bar = self.menuBar()

        self.file_menu = self.menu_bar.addMenu("File")
        self.file_menu.addAction('New')
        self.file_menu.addAction('Open')
        self.file_menu.addAction('Save')
        self.file_menu.addAction('Save as')

        self.view_menu = self.menu_bar.addMenu("View")
        self.view_menu.addAction('set Full Screen')
        self.view_menu.addAction('show Status Bar') 

        self.edit_menu = self.menu_bar.addMenu("Edit")
        self.edit_menu.addAction('Cut')
        self.edit_menu.addAction('Copy')
        self.edit_menu.addAction('Paste')
        self.edit_menu.addAction('Find')
        self.edit_menu.addAction('Replace') 

        self.help_menu = self.menu_bar.addMenu("Help")
        self.help_menu.addAction('Help')
        self.help_menu.addAction('About')

        # подключение к сокету
        self.user_text = ""

        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.client_socket.connect(('127.0.0.1', 12345))
        self.input_data = {}
        self.logger.info("<We are in UI: welcome. Connected to the socket>")
        self.feedback_widget = FeedbackWidget(start_hidden=False)
        
        # инициализация основы окончена, переход к инициализации содержимого
        self.initUI()

    def initUI(self):

        # создание трёх кнопок с надписями на них
        # для этого создаётся layout, и на нём размещаются кнопки
        layout = QGridLayout()

        button1 = QPushButton('Reflection/Transmition \n spectrum of \n 1D Photonic Crystall', self)
        button2 = QPushButton('MODEL 2 \n (in progress)', self)
        button3 = QPushButton('MODEL 3 \n (in progress)', self)

        button1.setFixedSize(140, 140)
        button2.setFixedSize(140, 140)
        button3.setFixedSize(140, 140)

        layout.addWidget(button1, 1, 0, 1, 1)
        layout.addWidget(button2, 1, 1, 1, 1)
        layout.addWidget(button3, 1, 2, 1, 1)

        widget = QWidget()
        widget.setLayout(layout)
        self.setCentralWidget(widget)

        # действия в случае нажатия на кнопки, в данный момент прописана только одна кнопка
        button1.clicked.connect(lambda: self.on_button_click("transfer_matrix_with_init_socket.py"))
        # button2.clicked.connect(lambda: self.on_button_click("transfer_matrix_with_init_socket.py"))
        # button3.clicked.connect(lambda: self.on_button_click("transfer_matrix_with_init_socket.py"))

    # метод, который запускается при нажатии кнопки "button1"
    def on_button_click(self, script_name):
        # скрытие кнопки, смена экрана
        button = self.sender()
        button.hide()

        # Запись в лог о выборе кнопки
        self.logger.info(f"Button '{button.text()}' clicked")

        # Запускаем скрипт
        subprocess.Popen(["pythonw", script_name])

        # читаем данные из JSON
        self.client_socket.send(b"transfer_matrix")
        init_data_from_socket = self.client_socket.recv(1024)
        self.init_data = json.loads(init_data_from_socket.decode())

        # здесь начинается отрисовка юи
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        self.showMaximized()

         # здесь начинается то что будет справа
        topRightPlainText = QFrame()
        bottomRightPlainText = QPlainTextEdit()
        bottomRightPlainText.setPlainText("Bottom Right Plain Text")

        self.rightVBox = QVBoxLayout()
        self.rightVBox.addWidget(topRightPlainText, 4)
        self.rightVBox.addWidget(self.feedback_widget, 1)
        # здесь заканчивается то, что будет справа

        # здесь начинается то, что будет слева
        self.leftScrollArea = QScrollArea()

        container = QWidget()
        layout = QVBoxLayout(self.central_widget)

        self.input_widgets = {}
        self.output_widgets = {}

        # создание полей в зависимости от параметров
        for param_name, param_info in self.init_data["input_parameters"].items():
            self.parameter_processing(param_name, param_info, layout)

        # выпадающее меню с выбором типа спектра
        spectrum_type_combo = labeledComboBoxWidget("Choose spectrum type:", self.init_data["output_options"]["spectrum_type"])
        self.output_widgets["spectrum_type"] = spectrum_type_combo
        layout.addWidget(spectrum_type_combo)
        
        # создание кнопки "Show Spectrum"
        button = QPushButton("Show Spectrum")
        button.clicked.connect(lambda: self.show_spectrum(topRightPlainText))
        layout.addWidget(button)
        # здесь заканчивается то, что будет слева

        # размещение всех созданных элементов на layout
        container.setLayout(layout)
        self.leftScrollArea.setWidgetResizable(True)
        self.leftScrollArea.setWidget(container)

        self.centralHBox = QHBoxLayout()
        self.centralHBox.addWidget(self.leftScrollArea)
        self.centralHBox.addLayout(self.rightVBox)

        self.central_widget.setLayout(self.centralHBox)

    # по заданным в файле .json параметрам создаются поля/выпадающие списки в View
    def parameter_processing(self, par_name: str, par_info: dict, layout):
        match par_info["type"]:
            case "int":
                int_input_widget = labeledQLineEditor(par_info["description"], par_info["type"])
                layout.addWidget(int_input_widget)
                self.input_widgets[par_name] = int_input_widget

            case "string":
                str_input_widget = labeledQLineEditor(par_info["description"], par_info["type"])
                layout.addWidget(str_input_widget)
                self.input_widgets[par_name] = str_input_widget

            case "float":
                float_input_widget = labeledQLineEditor(par_info["description"], par_info["type"])
                layout.addWidget(float_input_widget)
                self.input_widgets[par_name] = float_input_widget

            case "dynamic_list":
                dyn_wdg = dynamicQLineEditor("Enter number of layers in unit cell", "Layer")
                layout.addWidget(dyn_wdg)
                self.input_widgets[par_name] = dyn_wdg


    def show_spectrum(self, space):
        # Запись в лог о выборе кнопки Show Spectrum
        self.logger.info("Button 'Show Spectrum' clicked")

        for param_name, input_widget in self.input_widgets.items():
            self.logger.info(
                f"Parameter: {param_name}, value: {input_widget.get_value()}, type of value: {type(input_widget.get_value())}")

            val = input_widget.get_value()
            print(f"{param_name = }, {val = }")
            self.input_data[param_name] = val

        for param_name, output_widget in self.output_widgets.items():
            self.logger.info(
                f"Parameter: {param_name}, value: {output_widget.get_value()}, type of value: {type(output_widget.get_value())}")

            val = output_widget.get_value()
            print(f"{param_name = }, {val = }")
            self.input_data["output_option"] = val


        print(self.input_data)

        with open("input_data.json", "w") as file:
            json.dump(self.input_data, file)
        self.logger.info("Data written to input_data.json")
        input_msg = json.dumps(self.input_data)

        self.client_socket.send(bytes(input_msg, encoding='utf-8'))

        self.logger.info("Show Spectrum button processed")

        output_data_from_socket = self.client_socket.recv(1000000)
        output_data = json.loads(output_data_from_socket.decode())
        self.logger.info("Output data from kernel is received")
        self.logger.info(output_data)

        x_data = output_data['x']
        y_data = output_data['y']

        # Создаем новое окно с графиком и записываем в лог
        plot_window = PlotWindow(x_data, y_data)
        lay_help = QHBoxLayout()
        lay_help.addWidget(plot_window, 4)
        space.setLayout(lay_help)
        self.logger.info("Plot window created")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon("icon.jpg"))
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec())
