import numpy as np
from PyQt6.QtWidgets import QWidget, QVBoxLayout, QPushButton, QFileDialog, QApplication
import pyqtgraph as pg
from PyQt6.QtGui import QFont, QPen, QColor
from pyqtgraph.exporters import ImageExporter


class PlotWindow(QWidget):
    def __init__(self, x_data, y_data, title='Reflection spectrum', x_label='Wavelength, nm', y_label='Reflection coefficient', parent=None):
        super().__init__(parent)

        self.plot_widget = pg.PlotWidget(self)

        styles = {'color': '#000', 'font-size': '14px', 'font-weight': 'bold'}
        pg.setConfigOptions(antialias=True)
        self.plot_widget.setBackground('w')
        self.plot_widget.setTitle(title, **styles)

        # Настройка осей
        self.configure_axis(self.plot_widget.getAxis('left'), y_label)
        self.configure_axis(self.plot_widget.getAxis('bottom'), x_label)

        # Добавление сетки
        self.plot_widget.showGrid(x=True, y=True, alpha=0.9)

        layout = QVBoxLayout(self)
        layout.addWidget(self.plot_widget)

        # Кнопка для сохранения графика
        save_button = QPushButton('Save Graph as PNG', self)
        save_button.clicked.connect(self.save_graph)
        layout.addWidget(save_button)

        pen = pg.mkPen(color='b', width=2)
        self.plot_widget.plot(x_data, y_data, pen=pen)

    def configure_axis(self, axis, label):
        axis.setLabel(label)
        font = QFont('Arial', 10, QFont.Weight.Bold)
        axis.setTickFont(font)
        axis.setTextPen(QPen(QColor(0, 0, 0)))

    # сохранение графика в виде PNG в выбранном файле
    def save_graph(self):
        file_name, _ = QFileDialog.getSaveFileName(self, "Save Graph", "", "PNG Files (*.png)")
        if file_name:
            if not file_name.endswith('.png'):
                file_name += '.png'
            exporter = ImageExporter(self.plot_widget.plotItem)
            exporter.export(file_name)


if __name__ == "__main__":
    x = np.linspace(0, 1, 100)
    y = x * x
    app = QApplication([])
    widget = PlotWindow(x, y)
    widget.show()
    app.exec()
